#!/bin/env bats

load '../../bats-support/load'
load '../../bats-file/load'

function setup {
    STACK_INSTALL_ROOT=$(cd mail-sync && stack path --local-install-root)
    MAIL_SYNC="${STACK_INSTALL_ROOT}/bin/mail-sync"
    TEST_TMP_DIR="$(temp_make)"
    MAIL_SYNC_1="${MAIL_SYNC} --config ${TEST_TMP_DIR}/test_config"
    MAIL_SYNC_2="${MAIL_SYNC} --config ${TEST_TMP_DIR}/test_config2"
    mkdir "${TEST_TMP_DIR}/sync"
    mkdir "${TEST_TMP_DIR}/sync2"
    mkdir "${TEST_TMP_DIR}/mail"
    mkdir "${TEST_TMP_DIR}/mail2"
    cp test/data/test_config "${TEST_TMP_DIR}"
    cp test/data/test_config "${TEST_TMP_DIR}/test_config2"
    ed "${TEST_TMP_DIR}/test_config" << EOF
s|ROOT|${TEST_TMP_DIR}/mail
s|SYNC|${TEST_TMP_DIR}/sync
s|CMD|("${MAIL_SYNC}", ["--config", "${TEST_TMP_DIR}/test_config"])
w
q
EOF
    ed "${TEST_TMP_DIR}/test_config2" << EOF
s|ROOT|${TEST_TMP_DIR}/mail2
s|SYNC|${TEST_TMP_DIR}/sync2
s|CMD|("${MAIL_SYNC}", ["--config", "${TEST_TMP_DIR}/test_config2"])
w
q
EOF
    mkdir -p "${TEST_TMP_DIR}/mail/testMails/cur"
    mkdir -p "${TEST_TMP_DIR}/mail/testMails/new"
    mkdir -p "${TEST_TMP_DIR}/mail/testMails/tmp"
    cat <<EOF > ${TEST_TMP_DIR}/test_sendmail
#!/bin/sh
echo \$* > ${TEST_TMP_DIR}/uux.save
echo >> ${TEST_TMP_DIR}/uux.save
cat >> ${TEST_TMP_DIR}/uux.save
EOF
   chmod +x ${TEST_TMP_DIR}/test_sendmail
}

function teardown {
    temp_del "$TEST_TMP_DIR"
}

@test "uuxBasic" {
  run sh -c "${MAIL_SYNC_1} init test1"
  echo $status
  echo $output
  [ "$status" -eq 0 ]
  run sh -c "${MAIL_SYNC_1} syncAllMailBoxesInit"
  [ "$status" -eq 0 ]
  

  run sh -c "${MAIL_SYNC_2} init test2"
  UUID2=`${MAIL_SYNC_2} printUUID`
  run sh -c "${MAIL_SYNC_1} initialSyncFor ${UUID2} test2 > ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]
  ${MAIL_SYNC_1} showGlobalState
  [ "$status" -eq 0 ]
  
  # Peer #2 can accept the initial bundle
  run sh -c "${MAIL_SYNC_2} initialSyncFrom < ${TEST_TMP_DIR}/initial"
  [ "$status" -eq 0 ]

  run sh -c "echo Hello World | ${MAIL_SYNC_1} uux ${UUID2} -- ${TEST_TMP_DIR}/test_sendmail -f me@example.com"
  ${MAIL_SYNC_1} showGlobalState
  
  ed "${TEST_TMP_DIR}/test_config" << EOF
s|Nothing|Just ["${MAIL_SYNC}", "-c", "${TEST_TMP_DIR}/test_config2"]|
w
q
EOF
  ed "${TEST_TMP_DIR}/test_config2" << EOF
s|/usr/sbin/sendmail|${TEST_TMP_DIR}/test_sendmail|
w
q
EOF
  run sh -c "${MAIL_SYNC_1} update test2"
  cat ${TEST_TMP_DIR}/sync2/log
  ${MAIL_SYNC_2} showGlobalState
  cat ${TEST_TMP_DIR}/uux.save
  cmp - ${TEST_TMP_DIR}/uux.save <<'EOF'
-f me@example.com

Hello World
EOF
}
